-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to set up a timer to auto-loot certain items
--
local function NigglesDelayAutoLoot()
  -- Check the option is enabled
  if ((NigglesDBC.settings.UiAutoLootArch  == true) or
      (NigglesDBC.settings.UiAutoLootFish  == true) or
      (NigglesDBC.settings.UiAutoLootMoney == true)) then
    -- Create a timer to auto-loot certain items
    NigglesTimerCreate("AutoLoot", NigglesAutoLoot, nil, nil, 0.15, false);
  end

  return;
end

--
-- Function to delete an item in a bag or bank slot with a modified click.
-- The deletion is confirmed by calling the appropriate Blizzard function.
--
local function NigglesItemDelete(bagId, slotId)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiItemDelete == true) then
    -- Get info for the item clicked on
    local itemId = GetContainerItemID(bagId, slotId);

    -- Check the click isn't already handled
    if ((itemId) and
        (IsModifiedClick("NIGGLESITEMDELETE"))) then
      -- Pick up the item clicked on
      PickupContainerItem(bagId, slotId);

      -- Get the quality of the item clicked on
      local itemName, _, itemQuality = GetItemInfo(itemId);

      -- Trigger the standard delete item action
      UIParent_OnEvent(self, "DELETE_ITEM_CONFIRM", itemName, itemQuality);
    end
  end

  return;
end

--
-- Function to delete an item in a bag with a modified click.
--
local function NigglesBagItemClicked(self, button)
  -- Delete the item
  NigglesItemDelete(self:GetParent():GetID(), self:GetID());

  return;
end

--
-- Function to delete an item in a bank slot with a modified click.
--
local function NigglesBankItemClicked(self, button)
  -- Delete the item
  NigglesItemDelete(BANK_CONTAINER, self:GetID());

  return;
end

--
-- Function to restore the previously saved state of all group entries in the
-- 'Reputation' frame.
--
local function NigglesFactionsRestore()
  -- Local variables
  local isHeader;
  local factionName = "";
  local numFactions;

  -- Check the option is enabled
  if (NigglesDBC.settings.UiRestoreFaction == true) then
    -- Expand all the factions
    ExpandAllFactionHeaders();

    -- Process each visible faction, in reverse order
    numFactions = GetNumFactions();
    for index = numFactions, 1, -1 do
      -- Get the faction's info
      factionName, _, _, _, _, _, _, _, isHeader = GetFactionInfo(index);

      -- Restore the faction's state, if it's a header
      if ((factionName ~= nil) and
          (isHeader)) then
        if (NigglesDBC.faction[factionName]) then
          CollapseFactionHeader(index);
        end
      end
    end
  end

  return;
end

--
-- Function to save the current state of all group entries in the 'Reputation'
-- frame.
--
local function NigglesFactionsSave()
  -- Local variables
  local isHeader;
  local isCollapsed;
  local lastName = "";
  local factionName = "";
  local index = 1;

  -- Process each faction in turn
  repeat
    -- Save the name of the last faction
    lastName = factionName

    -- Get the faction's info
    factionName, _, _, _, _, _, _, _, isHeader, isCollapsed =
      GetFactionInfo(index);

    -- Make sure this isn't the same faction as the last iteration
    if ((factionName ~= nil) and
        (factionName ~= lastName) and
        (isHeader)) then
      -- Save the faction's current state
      NigglesDBC.faction[factionName] = isCollapsed;
    end

    -- Increment the index
    index = index+1;
  until ((factionName == nil) or (factionName == lastName));

  return;
end

--
-- Function to switch to the first page of a merchant's wares when the 'Prev'
-- button is right-clicked.
--
local function NigglesMerchantFirst(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Prev' button is enabled
    if ((button == "RightButton") and
        (MerchantPrevPageButton:IsEnabled())) then
      MerchantFrame.page = 1;
      MerchantFrame_Update();
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to switch to the last page of a merchant's wares when the 'Next'
-- button is right-clicked.
--
local function NigglesMerchantLast(self, button)
  -- Local Variables
  local numItems = GetMerchantNumItems();

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Next' button is enabled
    if ((button == "RightButton") and
        (MerchantNextPageButton:IsEnabled())) then
      -- Change to the last page
      MerchantFrame.page = math.ceil(numItems/MERCHANT_ITEMS_PER_PAGE);
      MerchantFrame_Update();
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to zoom the minimap all the way in when the button is
-- right-clicked.
--
local function NigglesMinimapZoomIn(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiMiniMapZoom == true) then
    -- Check the right button was clicked and the 'Zoom In' button is enabled
    if ((button == "RightButton") and
        (MinimapZoomIn:IsEnabled())) then
      MinimapZoomIn:Disable();
      MinimapZoomOut:Enable();
      PlaySound("igMinimapZoomIn");
      Minimap:SetZoom(Minimap:GetZoomLevels()-1);
    end
  end

  return;
end

--
-- Function to zoom the minimap all the way out when the button is
-- right-clicked.
--
local function NigglesMinimapZoomOut(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiMiniMapZoom == true) then
    -- Check the right button was clicked and the 'Zoom Out' button is enabled
    if ((button == "RightButton") and
        (MinimapZoomOut:IsEnabled())) then
      MinimapZoomIn:Enable();
      MinimapZoomOut:Disable();
      PlaySound("igMinimapZoomOut");
      Minimap:SetZoom(0);
    end
  end

  return;
end

--
-- Function to toggle Equipment Manager Ignore for a item slot.
-- This function implements functionality from the function
-- 'PaperDollFrameItemFlyoutButton_OnClick' in PaperDollFrame.lua.
--
local function NigglesPaperDollItemSlotClicked(self)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiEMIgnoreToggle == true) then
    -- Check the correct modifier key is down and Equipment Manager is shown
    if ((IsAltKeyDown()) and
        ((PaperDollEquipmentManagerPane:IsShown() and
         (PaperDollEquipmentManagerPane.selectedSetName)) or
         (GearManagerDialogPopup:IsShown())) and
         (self.popoutButton:IsShown())) then
      -- Get the ID of the slot
      local slotId = self:GetID();

      -- Toggle the ignoring of the slot
      if (EquipmentManagerIsSlotIgnoredForSave(slotId)) then
        EquipmentManagerUnignoreSlotForSave(slotId);
        self.ignored = nil;
      else
        EquipmentManagerIgnoreSlotForSave(slotId);
        self.ignored = true;
      end
      PaperDollItemSlotButton_Update(self);
      PaperDollFrameItemFlyout_Show(self);
      PaperDollEquipmentManagerPaneSaveSet:Enable();

      -- Update the slot and the flyout menu
      PaperDollItemSlotButton_Update(self);
      PaperDollFrameItemFlyout_Show(self);
    end
  end

  return;
end

--
-- Function to switch to the first page of the spellbook when the 'Prev'
-- button is right-clicked.
--
local function NigglesSpellBookFirst(self, button)
  -- Local variables
  local currentPage;

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Prev' button is enabled
    if ((button == "RightButton") and
        (SpellBookPrevPageButton:IsEnabled())) then
      -- Get the current page
      currentPage = SpellBook_GetCurrentPage();

      -- Check the spell book isn't already on the first page
      if (currentPage > 1) then
        -- Change the page number to page 2
        if (SpellBookFrame.bookType == BOOKTYPE_SPELL) then
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.selectedSkillLine] = 2;
        else
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.bookType] = 2;
        end

        -- Call the function for the Next button to change to the last page
        SpellBookPrevPageButton_OnClick();
      end
    end
  end

  return;
end

--
-- Function to switch to the last page of the spellbook when the 'Next'
-- button is right-clicked.
--
local function NigglesSpellBookLast(self, button)
  -- Local variables
  local currentPage;
  local maxPages;

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Next' button is enabled
    if ((button == "RightButton") and
        (SpellBookNextPageButton:IsEnabled())) then
      -- Get the current and maximum pages
      currentPage, maxPages = SpellBook_GetCurrentPage();

      -- Check the spellbook isn't already on the last page
      if (currentPage < maxPages) then
        -- Change the page number to the penultimate page
        if (SpellBookFrame.bookType == BOOKTYPE_SPELL) then
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.selectedSkillLine] = maxPages-1;
        else
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.bookType] = maxPages-1;
        end

        -- Call the function for the Next button to change to the last page
        SpellBookNextPageButton_OnClick();
      end
    end
  end

  return;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to automatically loot certain items
--
function NigglesAutoLoot()
  -- Local Variables
  local numLootItems = GetNumLootItems();
  local isLocked;
  local itemName;
  local itemLink;

  -- Check the loot frame is shown
  if (LootFrame:IsShown()) then
    -- Check if the loot is from fishing
    if ((NigglesDBC.settings.UiAutoLootFish == true) and
        (IsFishingLoot())) then
      -- Loot all items
      for slotIdx = 1, numLootItems do
        _, _, _, _, _, _, _, isLocked = GetLootSlotInfo(slotIdx);
        if (not isLocked) then
          LootSlot(slotIdx);
        end
      end
    else
      -- Loot any appropriate items
      for slotIdx = 1, numLootItems do
        _, itemName, _, _, _, _, _, isLocked = GetLootSlotInfo(slotIdx);
        itemLink = GetLootSlotLink(slotIdx);
        if (not isLocked) then
          if (LootSlotIsCoin(slotIdx)) then
            if (NigglesDBC.settings.UiAutoLootMoney == true) then
              LootSlot(slotIdx);
            end
          elseif ((NigglesDBC.settings.UiAutoLootArch == true) and
                  (((LootSlotIsCurrency(slotIdx)) and
                    (NigglesArchaeologyIsFragment(itemName))) or
                   ((LootSlotIsItem(slotIdx)) and
                    (NigglesArchaeologyIsKeystone(itemLink))))) then
            LootSlot(slotIdx);
          else
          end
        end
      end
    end
  end

  return;
end

--
-- Function to toggle all the class color checkboxes in the Chat 'General 
-- Config' panel
--
function NigglesChatConfigOnClick(frame)
  -- Local variables
  local checkBox;
  local checked = false;
  local checkIdx = 1;
  local frameName = frame:GetName();

  -- Check the option is enabled
  if (NigglesDBC.settings.UiChatClassColor == true) then
    -- Work out what state to set the checkboxes to 
    checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
    while (checkBox ~= nil) do
      -- Check if the checkbox is visible
      if ((checkBox:IsVisible()) and
          (not checkBox:GetChecked())) then
        checked = true;
        break;
      end
      
      -- Move on to the next checkbox
      checkIdx = checkIdx+1;
      checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
    end

    -- Process each of the class color checkboxes
    checkIdx = 1;
    checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
    while (checkBox ~= nil) do
      -- Check if the checkbox is visible
      if (checkBox:IsVisible()) then
        checkBox:func(checked);
        checkBox:SetChecked(checked);
      end
      
      -- Move on to the next checkbox
      checkIdx = checkIdx+1;
      checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
    end
  end

  return;
end

--
-- Function to set the targets for the buttons in the Raid panel
--
function NigglesRaidButtonsSetTargets()
  -- Local variables
  local button;
  local unitName;

  -- Check the player is in a raid
  if (GetNumRaidMembers() > 0) then
    -- Set the target for all of the buttons
    for buttonIdx = 1, MAX_RAID_MEMBERS do
      button = _G["RaidGroupButton"..buttonIdx];
      if (button ~= nil) then
        unitName = _G[button:GetName().."Name"]:GetText();
        if (InCombatLockdown() == nil) then
          if ((unitName ~= nil) and
              (NigglesDBC.settings.RaidsTarget == true)) then
            button:SetAttribute("type", "target");
            button:SetAttribute("unit", unitName);
          else
            button:SetAttribute("type", nil);
            button:SetAttribute("unit", nil);
          end
        else
          Niggles.updateRaid = true;
        end
      end
    end
  end

  return;
end

--
-- Function to hook into the standard UI
--
function NigglesHookUi()
  -- Hook functions into the Zoom buttons on the Minimap
  MinimapZoomIn:HookScript("OnMouseUp", NigglesMinimapZoomIn);
  MinimapZoomOut:HookScript("OnMouseUp", NigglesMinimapZoomOut);

  -- Hook functions into 'Reputation' frame to save/restore factions' states
  ReputationFrame:HookScript("OnHide", NigglesFactionsSave);
  ReputationFrame:HookScript("OnShow", NigglesFactionsRestore);

  -- Hook a function into 'Raid' frame to set the targets for the buttons
  RaidFrame:HookScript("OnShow", NigglesRaidButtonsSetTargets);

  -- Hook a function into 'Loot' frame to automatically loot certain items
  LootFrame:HookScript("OnShow", NigglesDelayAutoLoot);

  -- Hook the clicking of any container item button
  hooksecurefunc("ContainerFrameItemButton_OnModifiedClick",
    NigglesBagItemClicked);
  hooksecurefunc("BankFrameItemButtonGeneric_OnModifiedClick",
    NigglesBankItemClicked);

  -- Hook functions into the 'Prev' and 'Next' buttons in the 'Spellbook' frame
  SpellBookPrevPageButton:HookScript("OnMouseUp", NigglesSpellBookFirst);
  SpellBookNextPageButton:HookScript("OnMouseUp", NigglesSpellBookLast);

  -- Hook functions into the 'Prev' and 'Next' buttons in the 'Merchant' frame
  MerchantPrevPageButton:HookScript("OnMouseUp", NigglesMerchantFirst);
  MerchantNextPageButton:HookScript("OnMouseUp", NigglesMerchantLast);

  -- Hook a function into the equipment buttons
  hooksecurefunc("PaperDollItemSlotButton_OnModifiedClick",
    NigglesPaperDollItemSlotClicked);

  return;
end
